package com.zoniot.ccrc.controller.common;

import com.alibaba.fastjson.JSONObject;
import com.zoniot.ccrc.commom.model.AjaxMessage;
import com.zoniot.ccrc.commom.model.ResultStatus;
import com.zoniot.ccrc.commom.utils.TreeUtil;
import com.zoniot.ccrc.dao.AreaMapper;
import com.zoniot.ccrc.dto.AreaDto;
import com.zoniot.ccrc.dto.DeviceMeasuringPointDto;
import com.zoniot.ccrc.dto.DeviceTypeDto;
import com.zoniot.ccrc.dto.TreeDataDto;
import com.zoniot.ccrc.service.DeviceTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.time.LocalDateTime;
import java.util.List;

@Controller
@RequestMapping("common")
@Api(tags = "公共模块")
public class CommonController {


    @Autowired
    private DeviceTypeService deviceTypeService;

    @Autowired
    private AreaMapper areaMapper;

    @Value("${iot.server}")
    private String server;
    @Value("${iot.port}")
    private int port;




    @ResponseBody
    @GetMapping("/getDeviceTypeList")
    @ApiOperation(value = "获取系列（设备类型）列表")
    public AjaxMessage<List<DeviceTypeDto>> getDeviceTypeList(
            @ApiParam(value = "系统id", required = false) @RequestParam(required = false) Integer sysId,
            @ApiParam(value = "设备类型 1:传感器 2;网关设备 3:集中器 4:采集器", required = false) @RequestParam(required = false) Integer type
    ) {
        List<DeviceTypeDto> list = deviceTypeService.getDeviceTypeList(sysId, type);
        return new AjaxMessage<>(ResultStatus.OK, list);
    }

    @ResponseBody
    @GetMapping("/getTreeData")
    @ApiOperation(value = "获取厂商/系列/型号树形数据")
    public AjaxMessage<List<TreeDataDto>> getTreeData(
            @ApiParam(value = "系统id", required = false) @RequestParam(required = false) Integer sysId,
            @ApiParam(value = "类型 1:传感器 2;网关设备 3:集中器 4:采集器", required = false) @RequestParam(required = false) Integer type
    ) {
        List<TreeDataDto> list = deviceTypeService.getTreeData(sysId, type);
        return new AjaxMessage<>(ResultStatus.OK, list);
    }


    @ResponseBody
    @GetMapping("/getAllArea")
    @ApiOperation(value = "获取所有的省市区")
    public AjaxMessage<List<AreaDto>> getAllArea(
    ) {
        List<AreaDto> list = areaMapper.getAllArea();
        return new AjaxMessage<>(ResultStatus.OK, TreeUtil.getArea(list, 100000, 1));
    }

    @GetMapping("/getDeviceMeasuringPoint")
    @ApiOperation(value = "获取设备测点")
    public void getDeviceMeasuringPoint(
            @ApiParam(value = "设备类型id", required = true) @RequestParam(required = true) Integer deviceTypeId,
            HttpServletRequest httpServletRequest,
            HttpServletResponse httpServletResponse
    ) throws URISyntaxException, IOException {
        RestTemplate restTemplate=new RestTemplate();
        URI uri = new URI("http", null, server, port, httpServletRequest.getRequestURI(), httpServletRequest.getQueryString(), null);
        ResponseEntity<String> responseEntity = restTemplate.exchange(uri, HttpMethod.GET, null, String.class);

        httpServletResponse.setCharacterEncoding("UTF-8");
        httpServletResponse.setContentType("application/json;charset=utf-8");
        httpServletResponse.getWriter().write(responseEntity.getBody());
    }

}
