package com.zoniot.ccrc.dao;

import com.zoniot.ccrc.dto.GridUser;
import com.zoniot.ccrc.entity.Building;
import com.zoniot.ccrc.entity.Community;
import com.zoniot.ccrc.entity.Device;
import com.zoniot.ccrc.entity.GridManagement;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * (GridManagement)表数据库访问层
 *
 * @author hym
 * @since 2021-02-23 11:48:07
 */
@Mapper
public interface GridManagementMapper {

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    GridManagement selectById(Integer id);


    /**
     * 查询全部
     *
     * @return 对象列表
     */
    List<GridManagement> selectAll();

    /**
     * 通过实体作为筛选条件查询
     *
     * @param gridManagement 实例对象
     * @return 对象列表
     */
    List<GridManagement> selectList(GridManagement gridManagement);

    /**
     * 新增数据
     *
     * @param gridManagement 实例对象
     * @return 影响行数
     */
    int insert(GridManagement gridManagement);

    /**
     * 批量新增
     *
     * @param gridManagements 实例对象的集合
     * @return 影响行数
     */
    int batchInsert(@Param("gridManagements") List<GridManagement> gridManagements);

    /**
     * 修改数据
     *
     * @param gridManagement 实例对象
     * @return 影响行数
     */
    int update(GridManagement gridManagement);

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 影响行数
     */
    int deleteById(Integer id);

    /**
     * 查询总数据数
     *
     * @return 数据总数
     */
    int count();

    List<GridUser> selectGirdUserInfo(@Param("username") String username,
                                      @Param("orgId") Integer orgId,
                                      @Param("siteId") Integer siteId,
                                      @Param("userId") Integer userId);

    List<Community> getCommutityByOrg(@Param("ids") List<Integer> orgId);

    List<Building> getBuildingByCommutity(@Param("commutityId") Integer commutityId);

    List<Device> getDevices(@Param("buildingId") Integer buildingId,
                            @Param("userId") Integer userId, @Param("address") String address);

    GridManagement getByDeviceId(@Param("deviceId") Long deviceId);
}