package com.zoniot.ccrc.dto;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.zoniot.ccrc.entity.Permission;
import com.zoniot.ccrc.entity.Site;
import com.zoniot.ccrc.entity.User;
import lombok.Getter;
import lombok.Setter;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

@Getter
@Setter
public class LoginUser extends User implements UserDetails {

    private static final long serialVersionUID = -549522926597325135L;

    //用户类型 0：无 1:超级管理员 2:站点管理员 3：站点用户 4:网格管理员 5:社区管理员
    private Integer type;

    //站点id
    private Integer siteId;


    //客户Id
    private Integer customerId;

    //客户名称
    private String customerName;


    private Site site;

    //组织ID
    private  Integer orgId;


    //是否是站点管理员
    private Integer isSiteAdmin = 0;

    //不同站点对应的功能权限
    @JsonIgnore
    private List<Permission> permissionList;


    @JsonIgnore
    private String LoginToken;
    @JsonIgnore
    private String iotToken;
    private String defaultUserNumber;
    private String openid;

    @JsonIgnore
    private LocalDateTime expireTime;

    @JsonIgnore
    public boolean isExpried() {
        return LocalDateTime.now().isAfter(expireTime);
    }

    @Override
    @JsonIgnore
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return permissionList.parallelStream().filter(p -> !StringUtils.isEmpty(p.getPermission()))
                .map(p -> new SimpleGrantedAuthority(p.getPermission())).collect(Collectors.toSet());
    }

    @Override
    @JsonIgnore
    public boolean isAccountNonExpired() {
        return true;
    }

    @Override
    @JsonIgnore
    public boolean isAccountNonLocked() {
        return true;
    }

    @Override
    @JsonIgnore
    public boolean isCredentialsNonExpired() {
        return true;
    }

    @Override
    @JsonIgnore
    public boolean isEnabled() {
        return true;
    }
}
