package com.zoniot.ccrc.security.integration;

import com.zoniot.ccrc.security.mobile.MobileLoginAuthenticationToken;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Base64;

public class IntegrationLoginAuthenticationFilter extends AbstractAuthenticationProcessingFilter {
    // ~ Static fields/initializers
    // =====================================================================================

    public static final String SPRING_SECURITY_FORM_SOURCE_KEY = "source";
    public static final String SPRING_SECURITY_FORM_CODE_KEY = "code";

    private String sourceParameter = SPRING_SECURITY_FORM_SOURCE_KEY;
    private String codeParameter = SPRING_SECURITY_FORM_CODE_KEY;
    private boolean postOnly = true;

    // ~ Constructors
    // ===================================================================================================

    public IntegrationLoginAuthenticationFilter() {
        super(new AntPathRequestMatcher("/integration/auth/login", "POST"));
    }

    // ~ Methods
    // ========================================================================================================

    @Override
    public Authentication attemptAuthentication(HttpServletRequest request,
                                                HttpServletResponse response) throws AuthenticationException {
        if (postOnly && !request.getMethod().equals("POST")) {
            throw new AuthenticationServiceException(
                    "Authentication method not supported: " + request.getMethod());
        }

        String source = request.getParameter(sourceParameter);
        String code = request.getParameter(codeParameter);

        if (source == null) {
            source = "";
        }

        if (code == null) {
            code = "";
        }
        byte[] base64decodedBytes = Base64.getDecoder().decode(code);

        String mobile = new String(base64decodedBytes);
        String password = "";

        MobileLoginAuthenticationToken authRequest = new MobileLoginAuthenticationToken(mobile, password);

        // Allow subclasses to set the "details" property
        setDetails(request, authRequest);

        return this.getAuthenticationManager().authenticate(authRequest);
    }


    protected void setDetails(HttpServletRequest request, MobileLoginAuthenticationToken authRequest) {
        authRequest.setDetails(authenticationDetailsSource.buildDetails(request));
    }
}
