package com.zoniot.ccrc.service.impl;


import com.zoniot.ccrc.commom.utils.TreeUtil;
import com.zoniot.ccrc.commom.utils.UserUtil;
import com.zoniot.ccrc.dao.GridManagementMapper;
import com.zoniot.ccrc.dao.OrganizationMapper;
import com.zoniot.ccrc.dto.GridInfo;
import com.zoniot.ccrc.dto.GridUser;
import com.zoniot.ccrc.dto.LoginUser;
import com.zoniot.ccrc.dto.OrganizationDto;
import com.zoniot.ccrc.entity.Building;
import com.zoniot.ccrc.entity.Community;
import com.zoniot.ccrc.entity.Device;
import com.zoniot.ccrc.entity.GridManagement;
import com.zoniot.ccrc.service.GridManagementService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * (GridManagement表)服务实现类
 *
 * @author hym
 * @since 2021-02-23 11:48:08
 */
@Service("gridManagementService")
public class GridManagementServiceImpl implements GridManagementService {
    @Resource
    private GridManagementMapper gridManagementMapper;
    @Resource
    private OrganizationMapper organizationMapper;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public GridManagement selectById(Integer id) {
        return this.gridManagementMapper.selectById(id);
    }


    /**
     * 查询所有
     *
     * @return 实例对象的集合
     */
    @Override
    public List<GridManagement> selectAll() {
        return this.gridManagementMapper.selectAll();
    }

    /**
     * 根据条件查询
     *
     * @return 实例对象的集合
     */
    @Override
    public List<GridManagement> selectList(GridManagement gridManagement) {
        return this.gridManagementMapper.selectList(gridManagement);
    }

    /**
     * 新增数据
     *
     * @param gridInfo 实例对象
     * @return 实例对象
     */
    @Override
    public int insert(GridInfo gridInfo) {
        Integer userId = gridInfo.getUserId();
        List<Long> deviceIds = gridInfo.getDeviceIds();
        List<GridManagement> gridManagements=new ArrayList<>();
        LoginUser currentUser = UserUtil.getCurrentUser();
        deviceIds.forEach(deviceId->{
            GridManagement gridManagement=new GridManagement();
            gridManagement.setCreateBy(currentUser.getUsername());
            gridManagement.setCreateTime(new Date());
            gridManagement.setUserId(userId);
            gridManagement.setDeviceId(deviceId);
            gridManagement.setStatus(1);
            gridManagement.setIsDefault(0);
            gridManagements.add(gridManagement);
        });
        if(gridManagements.size()>0)
        gridManagementMapper.batchInsert(gridManagements);
        return 1;
    }

    /**
     * 批量新增
     *
     * @param gridManagements 实例对象的集合
     * @return 生效的条数
     */
    @Override
    public int batchInsert(List<GridManagement> gridManagements) {
        return this.gridManagementMapper.batchInsert(gridManagements);
    }

    /**
     * 修改数据
     *
     * @param gridManagement 实例对象
     * @return 实例对象
     */
    @Override
    public int update(GridManagement gridManagement) {
        LoginUser currentUser = UserUtil.getCurrentUser();
        gridManagement.setUpdateBy(currentUser.getUsername());
        gridManagement.setUpdateTime(new Date());
        return this.gridManagementMapper.update(gridManagement);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @Override
    public int deleteById(Integer id) {
        return this.gridManagementMapper.deleteById(id);
    }

    /**
     * 查询总数据数
     *
     * @return 数据总数
     */
    @Override
    public int count() {
        return this.gridManagementMapper.count();
    }

    @Override
    public List<GridUser> selectGirdUserInfo(String name, Integer orgId) {

        LoginUser currentUser = UserUtil.getCurrentUser();

        Integer siteId=currentUser.getSiteId();
        Integer userId=currentUser.getId();
        List<GridUser> users=gridManagementMapper.selectGirdUserInfo(name,orgId,siteId,userId);
        return users;
    }

    @Override
    public List<Community> getCommutityByOrg(Integer orgId) {
        List<OrganizationDto> list = organizationMapper.findList(UserUtil.getCurrentUser().getSiteId());
        List<Integer>ids=new ArrayList<>();

        TreeUtil.getOrganList(list,orgId,0,ids);
        ids.add(orgId);
        //TreeUtil.getOrganTree()
        List<Community> communities=gridManagementMapper.getCommutityByOrg(ids);
        return communities;
    }

    @Override
    public List<Building> getBuildingByCommutity(Integer commutityId) {
        List<Building>buildings= gridManagementMapper.getBuildingByCommutity( commutityId);
        return buildings;
    }

    @Override
    public List<Device> getDevices(Integer buildingId, Integer userId, String address) {
        List<Device> devices=gridManagementMapper.getDevices(buildingId,userId,address);
        return devices;
    }

    @Override
    public void deleteGridDevice(Integer id) {
        GridManagement gridManagement=new GridManagement();
        gridManagement.setId(id);
        gridManagement.setStatus(0);
        gridManagementMapper.update(gridManagement);
    }

}