package com.zoniot.ccrc.service.impl;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.zoniot.ccrc.commom.exception.ServiceException;
import com.zoniot.ccrc.commom.model.Pagination;
import com.zoniot.ccrc.commom.utils.UserUtil;
import com.zoniot.ccrc.commom.utils.Util;
import com.zoniot.ccrc.dao.*;
import com.zoniot.ccrc.dto.LoginUser;
import com.zoniot.ccrc.dto.SiteDto;
import com.zoniot.ccrc.entity.*;
import com.zoniot.ccrc.service.SiteService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;

import static com.google.common.collect.Lists.newArrayList;

@Slf4j
@Service
public class SiteServiceImpl implements SiteService {

    @Resource
    private SiteMapper siteMapper;
    @Resource
    private SiteUserMapper siteUserMapper;
    @Resource
    private PermissionMapper permissionMapper;
    @Resource
    private RolePermissionMapper rolePermissionMapper;
    @Resource
    private RoleMapper roleMapper;
    @Resource
    private UserRoleMapper userRoleMapper;
    @Resource
    private UserMapper userMapper;
    @Resource
    private SitePermissionMapper sitePermissionMapper;

    @Override
    public int insert(Site site) {
        return siteMapper.insert(site);
    }

    @Override
    public int insertSelective(Site site) {
        site.setStatus(1);
        site.setCreateBy(UserUtil.getUsername());
        site.setCreateDate(LocalDateTime.now());
        site.setUpdateBy(UserUtil.getUsername());
        site.setUpdateDate(LocalDateTime.now());
        return siteMapper.insertSelective(site);
    }

    @Override
    public int insertList(List<Site> sites) {
        return siteMapper.insertList(sites);
    }

    @Override
    public int updateByPrimaryKeySelective(Site site) {
        site.setUpdateBy(UserUtil.getUsername());
        site.setUpdateDate(LocalDateTime.now());
        return siteMapper.updateByPrimaryKeySelective(site);
    }

    @Override
    public Pagination<SiteDto> getPage(List<Integer> siteIds, String name, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<SiteDto> list = siteMapper.getList(siteIds, name);
        list.forEach(siteDto -> {
            siteDto.setUserIds(siteUserMapper.findBySiteId(siteDto.getId(),1));
        });
        return new Pagination<>(list);
    }

    @Override
    public List<SiteDto> getList() {
        LoginUser loginUser = UserUtil.getCurrentUser();
        if (loginUser.getType() == 1) {
            return siteMapper.getList(null, null);
        }else {
            List<Integer> siteIds = newArrayList();
            siteIds.add(loginUser.getSiteId());
            return siteMapper.getList(siteIds, null);
        }
    }

    @Override
    @Transactional
    public void add(SiteDto siteDto) {
        log.info("begin add site siteDto:" + JSON.toJSONString(siteDto));
        int result = siteMapper.findByNameUnique(null, siteDto.getName());
        if (result > 0) {
            throw new ServiceException(-900, "站点名称已经存在");
        }
        int CustomerIdResult = siteMapper.findByCustomerIdUnique(null, siteDto.getCustomerId());
        if (CustomerIdResult > 0) {
            throw new ServiceException(-900, "该客户的站点已经存在");
        }

        this.insertSelective(siteDto);

        //添加站点管理员角色及权限
        Role role = new Role();
        role.setSiteId(siteDto.getId());
        role.setName("站点管理员");
        role.setDescription("");
        role.setStatus(1);
        role.setType(1);
        role.setCreateBy(UserUtil.getUsername());
        role.setCreateDate(LocalDateTime.now());
        role.setUpdateBy(UserUtil.getUsername());
        role.setUpdateDate(LocalDateTime.now());
        roleMapper.insertSelective(role);


        //多个站点管理员
        siteDto.getUserIds().forEach(userId -> {
            //添加站点管理员
            SiteUser siteUser = new SiteUser();
            siteUser.setSiteId(siteDto.getId());
            siteUser.setUserId(userId);
            siteUser.setStatus(1);
            siteUser.setIsAdmin(1);
            siteUser.setType(2);
            siteUser.setCreateBy(UserUtil.getUsername());
            siteUser.setCreateDate(LocalDateTime.now());
            siteUser.setUpdateBy(UserUtil.getUsername());
            siteUser.setUpdateDate(LocalDateTime.now());
            siteUserMapper.insert(siteUser);


            //分配用户站点管理员角色
            UserRole userRole = new UserRole();
            userRole.setUid(userId);
            userRole.setRid(role.getId());
            userRole.setStatus(1);
            userRole.setCreateBy(UserUtil.getUsername());
            userRole.setCreateDate(LocalDateTime.now());
            userRole.setUpdateBy(UserUtil.getUsername());
            userRole.setUpdateDate(LocalDateTime.now());
            userRoleMapper.insertSelective(userRole);
        });




        // 3、添加站点权
        if (siteDto.getPermissionIds() != null && siteDto.getPermissionIds().size() > 0) {
            List<SitePermission> list = newArrayList();
            for (Integer permissionId : siteDto.getPermissionIds()) {
                SitePermission sitePermission = new SitePermission();
                sitePermission.setSiteId(siteDto.getId());
                sitePermission.setPermissionId(permissionId);
                sitePermission.setStatus(1);
                sitePermission.setCreateBy(UserUtil.getUsername());
                sitePermission.setCreateDate(LocalDateTime.now());
                sitePermission.setUpdateBy(UserUtil.getUsername());
                sitePermission.setUpdateDate(LocalDateTime.now());
                list.add(sitePermission);
            };
            sitePermissionMapper.batchInsert(list);

        }

        siteMapper.updateCustomerSiteId(siteDto.getId(),siteDto.getCustomerId());
        log.info("end add site");
    }

    @Override
    @Transactional
    public void edit(SiteDto siteDto) {
        log.info("begin edit site siteDto:" + JSON.toJSONString(siteDto));
        int result = siteMapper.findByNameUnique(siteDto.getId(), siteDto.getName());
        if (result > 0) {
            throw new ServiceException(-900, "站点名称已经存在");
        }

        int CustomerIdResult = siteMapper.findByCustomerIdUnique(siteDto.getId(), siteDto.getCustomerId());
        if (CustomerIdResult > 0) {
            throw new ServiceException(-900, "该客户的站点已经存在");
        }

        this.updateByPrimaryKeySelective(siteDto);


        //先删除用户站点管理员角色再重新分配
        Role role = roleMapper.findBySiteIdAndType(siteDto.getId(), 1);
        //userRoleMapper.deleteByRoleId(role.getId(), UserUtil.getCurrentUser().getUsername());


        //先删除旧的和新的站点管理员差集
        List<Integer> siteAdminUserIds = siteUserMapper.findBySiteId(siteDto.getId(),1);
        log.info("siteAdminUserIds={}",siteAdminUserIds);
        List<Integer> newSiteAdminUserIds = siteDto.getUserIds();
        List<Integer> deleteSiteAdminUserIds = Util.difference(siteAdminUserIds,newSiteAdminUserIds);
        log.info("newSiteAdminUserIds={}",newSiteAdminUserIds);
        if (deleteSiteAdminUserIds != null && deleteSiteAdminUserIds.size() > 0) {
            siteUserMapper.deleteBySiteIdAndUserIds(siteDto.getId(), deleteSiteAdminUserIds);
            userRoleMapper.deleteBySiteIdAndUserIds(siteDto.getId(), deleteSiteAdminUserIds);
        }


        //再添加新的和旧的管理员差集
        List<Integer> addSiteAdminUserIds = Util.difference(newSiteAdminUserIds,siteAdminUserIds);
        log.info("addSiteAdminUserIds={}",addSiteAdminUserIds);
        addSiteAdminUserIds.forEach(userId->{
                SiteUser siteUser = new SiteUser();
                siteUser.setSiteId(siteDto.getId());
                siteUser.setUserId(userId);
                siteUser.setStatus(1);
                siteUser.setIsAdmin(1);
                siteUser.setType(2);
                siteUser.setCreateBy(UserUtil.getUsername());
                siteUser.setCreateDate(LocalDateTime.now());
                siteUser.setUpdateBy(UserUtil.getUsername());
                siteUser.setUpdateDate(LocalDateTime.now());
                siteUserMapper.insert(siteUser);

                UserRole userRole = new UserRole();
                userRole.setUid(userId);
                userRole.setRid(role.getId());
                userRole.setStatus(1);
                userRole.setCreateBy(UserUtil.getUsername());
                userRole.setCreateDate(LocalDateTime.now());
                userRole.setUpdateBy(UserUtil.getUsername());
                userRole.setUpdateDate(LocalDateTime.now());
                userRoleMapper.insertSelective(userRole);


        });


        //3、先删除场景和权限再添加

        sitePermissionMapper.delBySiteId(siteDto.getId(), UserUtil.getUsername());

        if (siteDto.getPermissionIds() != null && siteDto.getPermissionIds().size() > 0) {
            List<SitePermission> list = newArrayList();

            for (Integer permissionId : siteDto.getPermissionIds()) {
                SitePermission sitePermission = new SitePermission();
                sitePermission.setSiteId(siteDto.getId());
                sitePermission.setPermissionId(permissionId);
                sitePermission.setStatus(1);
                sitePermission.setCreateBy(UserUtil.getUsername());
                sitePermission.setCreateDate(LocalDateTime.now());
                sitePermission.setUpdateBy(UserUtil.getUsername());
                sitePermission.setUpdateDate(LocalDateTime.now());
                list.add(sitePermission);
            };
            sitePermissionMapper.batchInsert(list);


        }
        siteMapper.deleteCustomerRelations(siteDto.getId());
        siteMapper.updateCustomerSiteId(siteDto.getId(),siteDto.getCustomerId());
        log.info("end edit site");
    }

    @Override
    @Transactional
    public void delete(Integer id) {
        log.info("begin delete site id:" + id);
        //删除站点
        siteMapper.delete(id, UserUtil.getCurrentUser().getUsername());

        //删除站点用户
        List<Integer> list = siteUserMapper.findBySiteId(id,null);
        //userMapper.batchDelectUser(list, UserUtil.getCurrentUser().getUsername());
        siteUserMapper.deleteBySiteId(id, UserUtil.getCurrentUser().getUsername(), null);
        siteMapper.deleteCustomerRelations(id);
        log.info("end delete site");
    }

    @Override
    public List<Customer> getCustomerList() {
        List<Customer> customers=siteMapper.getCustomerList();
        return customers;
    }

}
